<?php
namespace Yay_Currency\Engine\Calculate;

use Yay_Currency\Helpers\YayCurrencyHelper;
use Yay_Currency\Utils\SingletonTrait;


defined( 'ABSPATH' ) || exit;

class ShippingFlatRate extends \WC_Shipping_Method {
	use SingletonTrait;

	private $fee_cost       = 0;
	private $apply_currency = array();

	public function __construct() {
		$this->apply_currency = YayCurrencyHelper::detect_current_currency();
	}

	// Change currency when send mail end

	public function evaluate_cost( $sum, $args = array(), $caculate_checkout_fallback = false ) {
		// Add warning for subclasses.
		if ( ! is_array( $args ) || ! array_key_exists( 'qty', $args ) || ! array_key_exists( 'cost', $args ) ) {
			wc_doing_it_wrong( __FUNCTION__, '$args must contain `cost` and `qty` keys.', '4.0.1' );
		}

		include_once WC()->plugin_path() . '/includes/libraries/class-wc-eval-math.php';

		// Allow 3rd parties to process shipping cost arguments.
		$args           = apply_filters( 'woocommerce_evaluate_shipping_cost_args', $args, $sum, $this );
		$locale         = localeconv();
		$decimals       = array( wc_get_price_decimal_separator(), $locale['decimal_point'], $locale['mon_decimal_point'], ',' );
		$this->fee_cost = $args['cost'];
		// Expand shortcodes.
		add_shortcode( 'fee', array( $this, 'fee' ) );
		if ( $caculate_checkout_fallback ) {
			add_shortcode( 'fee_checkout_fallback', array( $this, 'fee_checkout_fallback' ) );
			$sum = str_replace( '[fee', '[fee_checkout_fallback', $sum );
		}
		$sum = do_shortcode(
			str_replace(
				array(
					'[qty]',
					'[cost]',
				),
				array(
					$args['qty'],
					$args['cost'],
				),
				$sum
			)
		);

		remove_shortcode( 'fee', array( $this, 'fee' ) );
		if ( $caculate_checkout_fallback ) {
			remove_shortcode( 'fee_checkout_fallback', array( $this, 'fee_checkout_fallback' ) );
		}
		// Remove whitespace from string.
		$sum = preg_replace( '/\s+/', '', $sum );

		// Remove locale from string.
		$sum = str_replace( $decimals, '.', $sum );

		// Trim invalid start/end characters.
		$sum = rtrim( ltrim( $sum, "\t\n\r\0\x0B+*/" ), "\t\n\r\0\x0B+-*/" );

		// Do the math.
		return $sum ? \WC_Eval_Math::evaluate( $sum ) : 0;
	}

	public function caculate_cost_fee( $atts = array(), $checkout_fallback = false ) {
		$min_fee                = $atts['min_fee'];
		$max_fee                = $atts['max_fee'];
		$caculate_with_currency = false;

		if ( 1 != YayCurrencyHelper::get_rate_fee( $this->apply_currency ) ) {
			if ( $checkout_fallback ) {
				$caculate_with_currency = true;
			} elseif ( ! YayCurrencyHelper::disable_fallback_option_in_checkout_page( $this->apply_currency ) ) {
				$caculate_with_currency = true;
			}
		}

		if ( $caculate_with_currency ) {
			$min_fee = YayCurrencyHelper::calculate_price_by_currency( $min_fee, false, $this->apply_currency );
			$max_fee = YayCurrencyHelper::calculate_price_by_currency( $max_fee, false, $this->apply_currency );
		}

		$calculated_fee = 0;

		if ( $atts['percent'] ) {
			$calculated_fee = $this->fee_cost * ( floatval( $atts['percent'] ) / 100 );
		}

		if ( $min_fee && $calculated_fee < $min_fee ) {
			$calculated_fee = $min_fee;
		}

		if ( $max_fee && $calculated_fee > $max_fee ) {
			$calculated_fee = $max_fee;
		}

		if ( $caculate_with_currency ) {
			$calculated_fee = (float) ( $calculated_fee / YayCurrencyHelper::get_rate_fee( $this->apply_currency ) );
		}

		return $calculated_fee;
	}

	public function fee( $atts ) {
		$atts           = shortcode_atts(
			array(
				'percent' => '',
				'min_fee' => '',
				'max_fee' => '',
			),
			$atts,
			'fee'
		);
		$calculated_fee = $this->caculate_cost_fee( $atts );
		return $calculated_fee;
	}

	public function fee_checkout_fallback( $atts ) {
		$atts           = shortcode_atts(
			array(
				'percent' => '',
				'min_fee' => '',
				'max_fee' => '',
			),
			$atts,
			'fee'
		);
		$calculated_fee = $this->caculate_cost_fee( $atts, true );
		return $calculated_fee;
	}

}
