<?php
namespace Yay_Currency\Engine\Compatibles;

use Yay_Currency\Utils\SingletonTrait;
use Yay_Currency\Helpers\YayCurrencyHelper;
use Yay_Currency\Helpers\FixedPriceHelper;
defined( 'ABSPATH' ) || exit;

class EnfoldTheme {

	use SingletonTrait;
	private $apply_currency = array();
	private $converted_currency;
	private $is_dis_checkout_diff_currency;
	public function __construct() {
		if ( 'enfold' !== wp_get_theme()->template ) {
			return;
		}

		$this->converted_currency            = YayCurrencyHelper::converted_currency();
		$this->apply_currency                = YayCurrencyHelper::get_apply_currency( $this->converted_currency );
		$is_checkout_different_currency      = get_option( 'yay_currency_checkout_different_currency', 0 );
		$this->is_dis_checkout_diff_currency = YayCurrencyHelper::is_dis_checkout_diff_currency( $is_checkout_different_currency, $this->apply_currency['status'] );

		if ( $this->is_dis_checkout_diff_currency ) {
			add_filter( 'woocommerce_get_price_html', array( $this, 'custom_price_html' ), 9999, 2 );
		}

	}

	public function custom_price_html( $price_html, $product ) {
		if ( is_checkout() ) {
			$currencies_data = YayCurrencyHelper::get_current_and_fallback_currency( $this->apply_currency, $this->converted_currency );
			if ( $currencies_data['current_currency']['currency'] == $currencies_data['fallback_currency']['currency'] ) {
				return $price_html;
			}

			$product_price              = $product->get_price();
			$original_product_subtotal  = YayCurrencyHelper::calculate_price_by_currency_html( $currencies_data['fallback_currency'], $product_price, 1 );
			$converted_product_subtotal = YayCurrencyHelper::calculate_price_by_currency_html( $currencies_data['current_currency'], $product_price, 1 );
			$product_fixed_price        = FixedPriceHelper::product_is_set_fixed_price_by_currency( $product, $this->apply_currency );
			if ( $product_fixed_price ) {
				$converted_product_subtotal = YayCurrencyHelper::format_price( $product_fixed_price );
			}
			$converted_product_subtotal_html = YayCurrencyHelper::converted_approximately_html( $converted_product_subtotal );
			$price_html                      = $original_product_subtotal . $converted_product_subtotal_html;
		}
		return $price_html;
	}

}
