<?php
namespace Yay_Currency\Engine;

use Yay_Currency\Utils\SingletonTrait;
use Yay_Currency\Helpers\Helper;
use Yay_Currency\Helpers\YayCurrencyHelper;
use Yay_Currency\Helpers\FixedPriceHelper;
use Yay_Currency\Helpers\TaxHelper;
use Yay_Currency\Helpers\SupportHelper;
use Yay_Currency\Engine\Calculate\ShippingFlatRate;

defined( 'ABSPATH' ) || exit;

class Hooks {
	use SingletonTrait;

	public function __construct() {
		// ADD FILTER PRIORITY
		add_filter( 'yay_currency_filters_priority', array( $this, 'get_filters_priority' ), 9, 1 );

		// ADD FILTER GET PRICE DEFAULT ON EVERY PAGE
		add_filter( 'yay_currency_get_cart_item_price', array( $this, 'get_cart_item_price' ), 10, 3 );
		add_filter( 'yay_currency_get_cart_subtotal', array( $this, 'get_cart_subtotal' ), 10, 2 );

		// PRODUCT SUBTOTAL, SUBTOTAL, TOTAL ---DISABLE CHECKOUT DIFF
		add_filter( 'yay_currency_checkout_converted_product_subtotal_fixed', array( $this, 'checkout_converted_product_subtotal_fixed' ), 10, 4 );
		add_filter( 'yay_currency_checkout_converted_subtotal_fixed', array( $this, 'checkout_converted_subtotal_fixed' ), 10, 2 );
		add_filter( 'yay_currency_checkout_converted_total_tax_fixed', array( $this, 'checkout_converted_total_tax_fixed' ), 10, 3 );
		add_filter( 'yay_currency_checkout_converted_total_fixed', array( $this, 'checkout_converted_total_fixed' ), 10, 2 );

		// GET CART SUBTOTAL BY FIXED
		add_filter( 'yay_currency_checkout_get_cart_subtotal_fixed', array( $this, 'checkout_get_cart_subtotal_fixed' ), 10, 2 );

		// DISCOUNT
		add_filter( 'yay_currency_checkout_formatted_percent_discount_price', array( $this, 'checkout_formatted_percent_discount_price' ), 10, 4 );
		add_filter( 'yay_currency_checkout_formatted_fixed_cart_or_product_discount_price', array( $this, 'checkout_formatted_fixed_cart_or_product_discount_price' ), 10, 5 );

		// ADD FILTER GET PRICE WITH CONDITIONS
		add_filter( 'yay_currency_get_price_with_conditions', array( $this, 'get_price_with_conditions' ), 10, 3 );

		// ADD FILTER GET PRICE EXCEPT CLASS PLUGINS
		add_filter( 'yay_currency_get_price_except_class_plugins', array( $this, 'get_price_except_class_plugins' ), 10, 3 );

		// Hooks for Shipping
		add_filter( 'yay_currency_formatted_shipping_flat_rate_fee', array( $this, 'formatted_shipping_flat_rate_fee' ), 10, 3 );

		// Custom Fixed Product Price
		add_filter( 'yay_currency_fixed_price_simple_product', array( $this, 'custom_fixed_price_simple_product' ), 10, 3 );
		add_filter( 'yay_currency_fixed_price_variable_product', array( $this, 'custom_fixed_price_variable_product' ), 10, 3 );
		add_filter( 'yay_currency_fixed_price_grouped_product', array( $this, 'custom_fixed_price_grouped_product' ), 10, 4 );

		add_filter( 'yay_currency_stripe_request_amount', array( $this, 'custom_stripe_request_amount' ), 10, 3 );

		add_action( 'yay_currency_redirect_to_url', array( $this, 'yay_currency_redirect_to_url' ), 10, 2 );

	}

	public function get_filters_priority( $priority ) {

		// Compatible with B2B Wholesale Suite, Price by Country, B2BKing
		if ( class_exists( 'B2bwhs' ) || class_exists( 'CBP_Country_Based_Price' ) || class_exists( 'B2bkingcore' ) || class_exists( 'BM' ) ) {
			$priority = 100000;
		}

		return $priority;

	}

	public function get_cart_item_price( $product_price, $cart_item, $apply_currency ) {
		$product_price = SupportHelper::get_product_price_by_cart_item( $cart_item, $apply_currency );
		return $product_price;
	}

	public function get_cart_subtotal( $cart_subtotal, $apply_currency ) {
		$cart_subtotal = SupportHelper::get_cart_subtotal( $apply_currency );
		return $cart_subtotal;
	}

	public function checkout_converted_product_subtotal_fixed( $converted_product_subtotal, $product, $apply_currency, $quantity ) {
		// Check if Product is sign_up fee
		$sign_up_fee = apply_filters( 'yay_currency_checkout_product_sign_up_fee', false, $product, $apply_currency );
		if ( $sign_up_fee ) {
			return $converted_product_subtotal;
		}

		if ( FixedPriceHelper::product_is_set_fixed_price_by_currency( $product, $apply_currency ) || 'disabled' !== $apply_currency['roundingType'] ) {
			$fixed_product_price    = (float) FixedPriceHelper::product_is_set_fixed_price_by_currency( $product, $apply_currency );
			$fixed_product_price    = apply_filters( 'yay_currency_get_product_fixed_price_3rd', $fixed_product_price, $product, $apply_currency );
			$fixed_product_subtotal = SupportHelper::get_product_subtotal_fixed_by_3rd_plugin( $fixed_product_price, $product, $quantity, $apply_currency );

			if ( $fixed_product_subtotal ) {
				$converted_product_subtotal = YayCurrencyHelper::calculate_custom_price_by_currency_html( $apply_currency, $fixed_product_subtotal );
			}

			return $converted_product_subtotal;
		}

		$converted_product_subtotal = apply_filters( 'yay_currency_checkout_converted_product_subtotal_with_3rd_plugin', $converted_product_subtotal, $product, $quantity, $apply_currency );
		return $converted_product_subtotal;

	}

	public function enable_fixed_price_or_rounding_currency( $apply_currency ) {
		$fixed_price = FixedPriceHelper::check_product_in_cart_fixed_price( $apply_currency );
		return $fixed_price || YayCurrencyHelper::enable_rounding_currency( $apply_currency );
	}

	public function checkout_converted_subtotal_fixed( $converted_subtotal, $apply_currency ) {
		$enable_fixed_or_rounding = $this->enable_fixed_price_or_rounding_currency( $apply_currency );
		if ( $enable_fixed_or_rounding ) {
			$subtotal_price_fixed = apply_filters( 'yay_currency_checkout_get_cart_subtotal_fixed', 0, $apply_currency );
			$converted_subtotal   = YayCurrencyHelper::calculate_custom_price_by_currency_html( $apply_currency, $subtotal_price_fixed );
			return $converted_subtotal;
		}

		$converted_subtotal = apply_filters( 'yay_currency_checkout_converted_subtotal_with_3rd_plugin', $converted_subtotal, $apply_currency );
		return $converted_subtotal;
	}

	public function checkout_converted_total_tax_fixed( $formatted_converted_tax_amount, $tax_rate_id, $apply_currency ) {
		$enable_fixed_or_rounding = $this->enable_fixed_price_or_rounding_currency( $apply_currency );
		if ( $enable_fixed_or_rounding ) {

			if ( count( WC()->cart->get_cart_contents() ) > 1 ) {
				$price = TaxHelper::get_price_by_tax_class( $apply_currency, $tax_rate_id );
			} else {
				$price = FixedPriceHelper::calculate_price_fixed_options( $apply_currency, 'total_tax_shipping_price_fixed' );
			}

			$formatted_converted_tax_amount = YayCurrencyHelper::calculate_custom_price_by_currency_html( $apply_currency, $price );

		}

		$formatted_converted_tax_amount = apply_filters( 'yay_currency_checkout_converted_total_tax_with_3rd_plugin', $formatted_converted_tax_amount, $tax_rate_id, $apply_currency );
		return $formatted_converted_tax_amount;

	}

	public function checkout_converted_total_fixed( $converted_total, $apply_currency ) {
		$enable_fixed_or_rounding = $this->enable_fixed_price_or_rounding_currency( $apply_currency );
		if ( $enable_fixed_or_rounding ) {
			$total_price_fixed = FixedPriceHelper::calculate_price_fixed_options( $apply_currency, 'total_price_fixed' );
			$converted_total   = YayCurrencyHelper::calculate_custom_price_by_currency_html( $apply_currency, $total_price_fixed );
			return $converted_total;
		}

		$converted_total = apply_filters( 'yay_currency_checkout_converted_total_with_3rd_plugin', $converted_total, $apply_currency );
		return $converted_total;
	}

	public function checkout_get_cart_subtotal_fixed( $subtotal, $apply_currency ) {
		$subtotal = SupportHelper::get_cart_subtotal( $apply_currency );
		// $subtotal = apply_filters( 'yay_currency_get_fixed_cart_subtotal_3rd_plugin', $subtotal, $apply_currency );
		return $subtotal;
	}

	public function checkout_formatted_percent_discount_price( $currencies_data, $coupon, $coupon_html, $discount_amount ) {
		$apply_currency           = $currencies_data['current_currency'];
		$discount_totals          = WC()->cart->get_coupon_discount_totals();
		$discount_price           = $discount_totals[ $coupon->get_code() ];
		$converted_discount_price = YayCurrencyHelper::calculate_price_by_currency( $discount_price, false, $apply_currency );
		$discount_amount_html     = YayCurrencyHelper::calculate_price_by_currency_html( $currencies_data['fallback_currency'], $discount_price );

		$converted_approximately = apply_filters( 'yay_currency_checkout_converted_approximately', true, $apply_currency );
		if ( ! $converted_approximately ) {
			return '-' . $discount_amount_html;
		}
		$enable_fixed_or_rounding = $this->enable_fixed_price_or_rounding_currency( $apply_currency );
		$formatted_discount_price = YayCurrencyHelper::format_price( $converted_discount_price );
		if ( $enable_fixed_or_rounding ) {
			$coupon_data  = $coupon->get_data();
			$products_ids = Helper::get_value_variable( $coupon_data['product_ids'] );
			if ( SupportHelper::woo_discount_rules_active() && $products_ids ) {
				$cart_subtotal_fixed      = SupportHelper::calculate_subtotal_apply_discount( $converted_discount_price, $discount_amount, $products_ids, $apply_currency );
				$formatted_discount_price = YayCurrencyHelper::format_price( $cart_subtotal_fixed );
			} else {
				$cart_subtotal_fixed      = FixedPriceHelper::calculate_price_fixed_options( $apply_currency, 'subtotal_price_fixed' );
				$cart_subtotal_fixed      = ( $cart_subtotal_fixed * $discount_amount ) / 100;
				$formatted_discount_price = YayCurrencyHelper::format_price( $cart_subtotal_fixed );
			}
		}
		$formatted_discount_price_html = YayCurrencyHelper::converted_approximately_html( $formatted_discount_price );
		$custom_coupon_html            = '-' . $discount_amount_html . $formatted_discount_price_html . substr( $coupon_html, strpos( $coupon_html, '<a' ) ) . '';
		return $custom_coupon_html;
	}

	public function checkout_formatted_fixed_cart_or_product_discount_price( $currencies_data, $coupon, $coupon_html, $discount_type, $discount_amount ) {
		$apply_currency = $currencies_data['current_currency'];

		if ( 'fixed_product' === $discount_type ) {
			$discount_totals = WC()->cart->get_coupon_discount_totals();
			$discount_amount = $discount_totals[ $coupon->get_code() ];
		}

		$converted_discount_amount = YayCurrencyHelper::calculate_price_by_currency( $discount_amount, false, $apply_currency );
		$formatted_discount_amount = YayCurrencyHelper::format_price( $converted_discount_amount );
		$discount_amount_html      = YayCurrencyHelper::calculate_price_by_currency_html( $currencies_data['fallback_currency'], $discount_amount );
		$converted_approximately   = apply_filters( 'yay_currency_checkout_converted_approximately', true, $apply_currency );

		if ( ! $converted_approximately ) {
			return '-' . $discount_amount_html;
		}

		$enable_fixed_or_rounding = $this->enable_fixed_price_or_rounding_currency( $apply_currency );
		if ( $enable_fixed_or_rounding ) {
			$cart_subtotal_fixed       = FixedPriceHelper::calculate_price_fixed_options( $apply_currency, 'subtotal_price_fixed_coupon' );
			$cart_subtotal_fixed       = $cart_subtotal_fixed > $converted_discount_amount ? $converted_discount_amount : $cart_subtotal_fixed;
			$formatted_discount_amount = YayCurrencyHelper::format_price( $cart_subtotal_fixed );
		}
		$formatted_discount_amount_html = YayCurrencyHelper::converted_approximately_html( $formatted_discount_amount );
		$formatted_coupon_html          = '-' . $discount_amount_html . $formatted_discount_amount_html . substr( $coupon_html, strpos( $coupon_html, '<a' ) ) . '';
		return $formatted_coupon_html;
	}

	public function get_price_with_conditions( $price, $product, $apply_currency ) {
		// YayExtra , YayPricing

		$is_ydp_adjust_price = false;
		$caculate_price      = YayCurrencyHelper::calculate_price_by_currency( $price, false, $apply_currency );

		if ( class_exists( '\YayPricing\FrontEnd\ProductPricing' ) ) {
			$is_ydp_adjust_price = apply_filters( 'ydp_check_adjust_price', false );
		}

		if ( class_exists( '\YayPricing\FrontEnd\ProductPricing' ) && $is_ydp_adjust_price ) {
			return $caculate_price;
		}

		$price_3rd_plugin = apply_filters( 'yay_currency_product_price_3rd_with_condition', false, $product );
		return $price_3rd_plugin;

	}

	public function get_price_except_class_plugins( $price, $product, $apply_currency ) {
		if ( class_exists( '\BM_Conditionals' ) ) {
			$group_id = \BM_Conditionals::get_validated_customer_group();
			if ( false !== $group_id ) {
				return $price;
			}
		}
		$calculate_price      = YayCurrencyHelper::calculate_price_by_currency( $price, false, $apply_currency );
		$except_class_plugins = array(
			'WC_Measurement_Price_Calculator',
			'\WP_Grid_Builder\Includes\Plugin',
			'WCPA', // Woocommerce Custom Product Addons
			'\Acowebs\WCPA\Main', // Woocommerce Custom Product Addons
			'WoonpCore', // Name Your Price for WooCommerce
			'Webtomizer\\WCDP\\WC_Deposits', // WooCommerce Deposits
			'\WC_Product_Price_Based_Country', // Price Per Country
			'\JET_APB\Plugin', // Jet Appointments Booking
		);
		$except_class_plugins = apply_filters( 'yay_currency_except_class_plugin', $except_class_plugins );
		foreach ( $except_class_plugins as $class ) {
			if ( class_exists( $class ) ) {
				return $calculate_price;
			}
		}
		return false;
	}

	public function formatted_shipping_flat_rate_fee( $formatted_shipping_fee, $method, $apply_currency ) {
		if ( 'flat_rate' == $method->method_id ) {
			$shipping               = new \WC_Shipping_Flat_Rate( $method->instance_id );
			$yay_shipping_flat_rate = new ShippingFlatRate();
			$cost                   = $shipping->get_option( 'cost' );
			if ( ! empty( $cost ) && ! is_numeric( $cost ) && FixedPriceHelper::check_product_in_cart_fixed_price( $apply_currency ) ) {
				$args                   = array(
					'qty'  => FixedPriceHelper::get_package_item_qty(),
					'cost' => apply_filters( 'yay_currency_checkout_get_cart_subtotal_fixed', 0, $apply_currency ),
				);
				$cost_caculate_again    = $yay_shipping_flat_rate->evaluate_cost( $cost, $args, true );
				$converted_shipping_fee = YayCurrencyHelper::calculate_price_by_currency( $cost_caculate_again, false, $apply_currency );
				$formatted_shipping_fee = YayCurrencyHelper::format_price( $converted_shipping_fee );
			}
		}
		return $formatted_shipping_fee;
	}

	public function custom_fixed_price_simple_product( $price, $product, $apply_currency ) {
		// WooCommerce Product Bundles plugin
		if ( 'bundle' === $product->get_type() ) {
			return $price;
		}

		$custom_fixed_prices = ! empty( get_post_meta( $product->get_ID(), 'yay_currency_custom_fixed_prices', true ) ) ? get_post_meta( $product->get_ID(), 'yay_currency_custom_fixed_prices', true ) : null;

		if ( $custom_fixed_prices && ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['price'] ) ) {
			if ( ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['regular_price'] ) ) {
				$formatted_custom_fixed_regular_price = YayCurrencyHelper::format_price( $custom_fixed_prices[ $apply_currency['currency'] ]['regular_price'] );
			} else {
				$formatted_custom_fixed_regular_price = YayCurrencyHelper::format_price( $product->get_regular_price() );
			}
			if ( $product->is_on_sale() ) {
				if ( ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['sale_price'] ) ) {
					$formatted_custom_fixed_sale_price = YayCurrencyHelper::format_price( $custom_fixed_prices[ $apply_currency['currency'] ]['sale_price'] );
				} else {
					$formatted_custom_fixed_sale_price = YayCurrencyHelper::format_price( $product->get_sale_price() );
				}
				$price = '<del class="strike">' . $formatted_custom_fixed_regular_price . '</del> 
				<ins class="highlight">' . $formatted_custom_fixed_sale_price . '</ins>';
			} else {
				$price = '<ins class="highlight">' . $formatted_custom_fixed_regular_price . '</ins>';
			}
		}

		$price = apply_filters( 'yay_currency_fixed_price_simple_product_convented', $price, $product, $apply_currency );
		return $price;
	}

	public function custom_fixed_price_variable_product( $price, $product, $apply_currency ) {
		$min_price     = $product->get_variation_price( 'min', true );
		$max_price     = $product->get_variation_price( 'max', true );
		$variation_ids = $product->get_children();

		foreach ( $variation_ids as $id ) {
			$custom_fixed_prices = get_post_meta( $id, 'yay_currency_custom_fixed_prices', true );
			if ( $custom_fixed_prices && ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['price'] ) ) {
				if ( $min_price == $max_price ) {
					$min_price = min( $min_price, $custom_fixed_prices[ $apply_currency['currency'] ]['price'] );
				} else {
					$min_price = max( $min_price, $custom_fixed_prices[ $apply_currency['currency'] ]['price'] );
				}
				$max_price     = max( $max_price, $custom_fixed_prices[ $apply_currency['currency'] ]['price'] );
				$sale_price    = ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['sale_price'] ) ? $custom_fixed_prices[ $apply_currency['currency'] ]['sale_price'] : '';
				$regular_price = ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['regular_price'] ) ? $custom_fixed_prices[ $apply_currency['currency'] ]['regular_price'] : '';
			}
		}

		if ( ! isset( $sale_price ) && ! isset( $regular_price ) ) {
			return $price;
		}

		$formatted_min_price = YayCurrencyHelper::format_price( $min_price );
		$formatted_max_price = YayCurrencyHelper::format_price( $max_price );

		if ( $min_price == $max_price ) {
			if ( ! empty( $sale_price ) && ! empty( $regular_price ) ) {
				$price = wc_format_sale_price( $regular_price, $sale_price );
			} else {
				$price = $formatted_min_price;
			}
		} else {
			$price = $formatted_min_price . ' - ' . $formatted_max_price;
		}

		$price = apply_filters( 'yay_currency_fixed_price_variable_product_convented', $price, $product, $apply_currency );
		return $price;

	}

	public function custom_fixed_price_grouped_product( $price, $product, $child_prices, $apply_currency ) {
		$min_price          = min( $child_prices );
		$max_price          = max( $child_prices );
		$linked_product_ids = $product->get_children();
		$fixed_prices_array = array();

		foreach ( $linked_product_ids as $id ) {
			$custom_fixed_prices = get_post_meta( $id, 'yay_currency_custom_fixed_prices', true );
			if ( $custom_fixed_prices && ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['price'] ) ) {
				array_push( $fixed_prices_array, $custom_fixed_prices[ $apply_currency['currency'] ]['price'] );
			}
		}

		if ( ! empty( $fixed_prices_array ) ) {
			$min_price = min( $fixed_prices_array );
			$max_price = max( $fixed_prices_array );
		}

		$formatted_min_price = YayCurrencyHelper::format_price( $min_price );
		$formatted_max_price = YayCurrencyHelper::format_price( $max_price );

		$price = $min_price == $max_price ? $formatted_min_price : $formatted_min_price . ' - ' . $formatted_max_price;
		$price = apply_filters( 'yay_currency_fixed_price_grouped_product_convented', $price, $product, $child_prices, $apply_currency );

		return $price;

	}

	public function custom_stripe_request_amount( $request, $api, $apply_currency ) {
		global $wpdb;
		if ( isset( $request['currency'] ) && isset( $request['metadata'] ) && isset( $request['metadata']['order_id'] ) ) {
			$array_zero_decimal_currencies = array(
				'BIF',
				'CLP',
				'DJF',
				'GNF',
				'JPY',
				'KMF',
				'KRW',
				'MGA',
				'PYG',
				'RWF',
				'UGX',
				'VND',
				'VUV',
				'XAF',
				'XOF',
				'XPF',
			);
			if ( in_array( strtoupper( $request['currency'] ), $array_zero_decimal_currencies ) ) {
				$orderID = $request['metadata']['order_id'];

				$result = $wpdb->get_var(
					$wpdb->prepare(
						"SELECT meta_value FROM {$wpdb->postmeta} WHERE (post_id = %d AND meta_key = '_order_total')",
						$orderID
					)
				);

				if ( empty( $result ) ) {
					return $request;
				}

				$order_total = $result;

				$request['amount'] = (int) $order_total;
			}
		}
		return $request;
	}

	public function yay_currency_redirect_to_url( $current_url, $currency_ID ) {
		$current_currency = YayCurrencyHelper::get_currency_by_ID( $currency_ID );
		$current_url      = add_query_arg( array( 'yay-currency' => $current_currency['currency'] ), $current_url );
		if ( wp_safe_redirect( $current_url ) ) {
			exit;
		}

	}

}
