<?php
namespace Yay_Currency\Helpers;

use Yay_Currency\Utils\SingletonTrait;
use Yay_Currency\Helpers\YayCurrencyHelper;
use Yay_Currency\Helpers\FixedPriceHelper;
class SupportHelper {

	use SingletonTrait;

	protected function __construct() {}

	public static function get_price_options_by_3rd_plugin( $product ) {
		$price_options = apply_filters( 'yay_currency_price_options', 0, $product );
		return $price_options;
	}

	public static function get_product_subtotal_fixed_by_3rd_plugin( $fixed_product_price, $product, $quantity, $apply_currency ) {
		$fixed_product_price          = apply_filters( 'yay_currency_get_fixed_product_price_3rd_plugin', $fixed_product_price, $product, $apply_currency );
		$fixed_product_subtotal_price = $fixed_product_price * $quantity;
		return apply_filters( 'yay_currency_get_fixed_product_subtotal_price_3rd_plugin', $fixed_product_subtotal_price, $product, $apply_currency );

	}

	public static function product_is_set_fixed_price( $apply_currency, $product_id ) {
		$price = false;
		if ( FixedPriceHelper::is_set_fixed_price() ) {
			$custom_fixed_prices = get_post_meta( $product_id, 'yay_currency_custom_fixed_prices', true );
			if ( ! empty( $custom_fixed_prices ) && isset( $custom_fixed_prices[ $apply_currency['currency'] ]['price'] ) ) {
				if ( ! empty( $custom_fixed_prices[ $apply_currency['currency'] ]['price'] ) ) {
					$sale_price_original_currency = get_post_meta( $product_id, '_sale_price', true );
					if ( ! empty( $sale_price_original_currency ) ) {
						$price = $custom_fixed_prices[ $apply_currency['currency'] ]['sale_price'] ? (float) $custom_fixed_prices[ $apply_currency['currency'] ]['sale_price'] : $price;
					} else {
						$price = $custom_fixed_prices[ $apply_currency['currency'] ]['regular_price'] ? (float) $custom_fixed_prices[ $apply_currency['currency'] ]['regular_price'] : $price;
					}
				}
			}
		}
		return $price;
	}

	public static function get_product_price( $product_id, $apply_currency = false ) {
		$_product      = wc_get_product( $product_id );
		$product_price = $_product->get_price( 'edit' );
		if ( $apply_currency ) {
			$product_price = YayCurrencyHelper::calculate_price_by_currency( $product_price, false, $apply_currency );
			$product_price = FixedPriceHelper::get_price_fixed_by_apply_currency( $_product, $product_price, $apply_currency );
		}
		return $product_price;
	}

	public static function woo_discount_rules_active() {
		return apply_filters( 'yay_currency_active_woo_discount_rules', false );
	}

	// Get product price by Woo Discount Rules plugin
	public static function get_product_fixed_price_by_discount_type( $price, $product, $apply_currency ) {
		//Support a few plugin like Extra, Add-ons
		$discount_type = isset( $product->awdr_product_get_discount_type ) ? $product->awdr_product_get_discount_type : false;
		if ( $discount_type ) {
			$discount_value = $product->awdr_product_get_discount_value;
			$quantity       = $product->awdr_product_get_quantity;
			switch ( $discount_type ) {
				case 'fixed_price':
					$price = YayCurrencyHelper::calculate_price_by_currency( $discount_value, false, $apply_currency ) * $quantity;
					break;
				case 'flat':
					$price = ( $price - YayCurrencyHelper::calculate_price_by_currency( $discount_value, false, $apply_currency ) ) * $quantity;
					break;
				default:
					$price = ( $price - ( $discount_value / 100 ) * floatval( $price ) ) * $quantity;
					break;
			}
		}

		return $price;
	}

	// Get original price by Woo Discount Rules plugin
	public static function get_product_price_default_by_discount_type( $price, $product ) {
		//Support a few plugin like Extra, Add-ons
		$discount_type = isset( $product->awdr_product_get_discount_type ) ? $product->awdr_product_get_discount_type : false;
		if ( $discount_type ) {
			$discount_value = $product->awdr_product_get_discount_value;
			switch ( $discount_type ) {
				case 'fixed_price':
					$price = $discount_value;
					break;
				case 'flat':
					$price = ( $price - $discount_value );
					break;
				default:
					$price = ( $price - ( $discount_value / 100 ) * floatval( $price ) );
					break;
			}
		}

		return $price;
	}

	// SUPPORT FOR Woo Discount Rules PRO
	public static function calculate_discount_from() {
		$calculate_discount_from = 'sale_price';
		if ( self::woo_discount_rules_active() && class_exists( '\Wdr\App\Controllers\DiscountCalculator' ) ) {
			$calculate_discount_from = \Wdr\App\Controllers\DiscountCalculator::$config->getConfig( 'calculate_discount_from', 'sale_price' );
		}
		return $calculate_discount_from;
	}

	public static function get_original_price_apply_discount_pro( $product_id ) {
		$calculate_discount_from = self::calculate_discount_from();
		if ( 'sale_price' === $calculate_discount_from ) {
			$original_price = (float) get_post_meta( $product_id, '_sale_price', true );
		} else {
			$original_price = (float) get_post_meta( $product_id, '_regular_price', true );
		}
		return (float) $original_price;
	}

	public static function product_fixed_price_apply_discount_pro_by_currency( $price_fixed, $product, $apply_currency ) {
		if ( FixedPriceHelper::is_set_fixed_price() ) {
			$custom_fixed_prices = get_post_meta( $product->get_id(), 'yay_currency_custom_fixed_prices', true );
			$currency_code       = isset( $apply_currency['currency'] ) ? $apply_currency['currency'] : '';
			if ( ! empty( $custom_fixed_prices ) && isset( $custom_fixed_prices[ $currency_code ] ) && $custom_fixed_prices[ $currency_code ] ) {
				$calculate_discount_from = self::calculate_discount_from();
				$regular_price           = Helper::get_value_variable( $custom_fixed_prices[ $currency_code ]['regular_price'], $price_fixed );
				if ( 'sale_price' === $calculate_discount_from ) {
					$price_fixed = Helper::get_value_variable( $custom_fixed_prices[ $currency_code ]['sale_price'], $regular_price );
				} else {
					$price_fixed = $regular_price;
				}
			}
		}
		return (float) $price_fixed;
	}

	public static function calculate_subtotal_apply_discount( $price, $discount_amount, $products_ids, $apply_currency ) {
		$subtotal = apply_filters( 'yay_currency_discount_rules_get_cart_subtotal_apply_coupon_as_cart_rule', $price, $discount_amount, $products_ids, $apply_currency );
		return $subtotal;
	}

	// Get Original Price by Cart Item
	public static function get_product_price_by_cart_item( $cart_item, $apply_currency ) {
		$product_id    = $cart_item['variation_id'] ? $cart_item['variation_id'] : $cart_item['product_id'];
		$product_price = self::get_product_price( $product_id, $apply_currency );
		$product_price = apply_filters( 'yay_currency_get_product_price_by_cart_item', $product_price, $cart_item, $apply_currency );
		$price_options = apply_filters( 'yay_currency_get_price_options_by_cart_item', 0, $cart_item, $product_id, $product_price, $apply_currency );
		return $price_options ? $product_price + $price_options : $product_price;
	}

	public static function get_cart_subtotal( $apply_currency = false ) {
		$subtotal      = 0;
		$cart_contents = WC()->cart->get_cart_contents();
		foreach ( $cart_contents  as $key => $cart_item ) {
			$product_price    = self::get_product_price_by_cart_item( $cart_item, $apply_currency );
			$product_subtotal = $product_price * $cart_item['quantity'];
			$subtotal         = $subtotal + $product_subtotal;
		}

		return $subtotal;
	}

}
