<?php
namespace Yay_Currency\Helpers;

use Yay_Currency\Utils\SingletonTrait;
use Yay_Currency\Helpers\YayCurrencyHelper;
use Yay_Currency\Helpers\FixedPriceHelper;
use Yay_Currency\Engine\Calculate\ShippingFlatRate;
class TaxHelper {

	use SingletonTrait;

	protected function __construct() {}

	public static function get_tax_info_by_product( $_product ) {
		$tax_rates   = \WC_Tax::get_rates( $_product->get_tax_class() );
		$rateId      = key( $tax_rates );
		$tax_rate    = \WC_Tax::_get_tax_rate( $rateId );
		$tax_class   = isset( $tax_rate['tax_rate_class'] ) && ! empty( $tax_rate['tax_rate_class'] ) ? $tax_rate['tax_rate_class'] : false;
		$tax_details = array(
			'tax_status'   => $_product->get_tax_status(),
			'tax_class'    => $tax_class ? $tax_class : 'standard-rate',
			'tax_rate_id'  => $rateId,
			'tax_value'    => \WC_Tax::get_rate_percent_value( $rateId ),
			'tax_rate'     => $tax_rate,
			'tax_code'     => \WC_Tax::get_rate_code( $rateId ),
			'tax_compound' => \WC_Tax::is_compound( $rateId ),
		);
		return $tax_details;
	}

	public static function get_taxes_in_cart( $apply_currency, $cart_contents ) {
		$taxes    = array();
		$standard = false;
		foreach ( $cart_contents  as $key => $cart_item ) {
			$product_id = $cart_item['variation_id'] ? $cart_item['variation_id'] : $cart_item['product_id'];
			$wc_product = wc_get_product( $product_id );
			$tax_info   = self::get_tax_info_by_product( $wc_product );
			$tax_class  = $tax_info['tax_class'];

			$products = array();

			if ( 'standard-rate' === $tax_class ) {
				$standard = true;
			}

			$quantity   = $cart_item['quantity'];
			$tax_status = $tax_info['tax_status'];
			$subtotal   = 0;

			if ( 'taxable' === $tax_status ) {
				$product_price = SupportHelper::get_product_price_by_cart_item( $cart_item, $apply_currency );
				$subtotal      = $subtotal + ( $product_price * $quantity );
			}

			if ( isset( $taxes[ $tax_class ] ) ) {
				$merge_subtotal                = $taxes[ $tax_class ]['subtotal'] + $subtotal;
				$quantity_product              = $taxes[ $tax_class ]['quantity_product'] + $quantity;
				$merge_products                = $taxes[ $tax_class ]['products'];
				$merge_products[ $product_id ] = array(
					'product_id' => $product_id,
					'quantity'   => $quantity,
					'subtotal'   => $subtotal,
				);
			}

			if ( 'none' != $tax_status && $tax_info['tax_value'] > 0 ) {
				$products[ $product_id ] = array(
					'product_id' => $product_id,
					'quantity'   => $quantity,
					'subtotal'   => $subtotal,
				);
				$taxes[ $tax_class ]     = array(
					'tax_info'         => $tax_info,
					'tax_rate'         => $tax_info['tax_rate'],
					'tax_class'        => $tax_class,
					'subtotal'         => isset( $merge_subtotal ) ? $merge_subtotal : $subtotal,
					'products'         => isset( $merge_products ) ? $merge_products : $products,
					'quantity_product' => isset( $quantity_product ) ? $quantity_product : $quantity,
				);
			}
		}

		$all_rates_class = \WC_Tax::get_tax_classes();
		$tax_apply       = ! $all_rates_class ? 'standard-rate' : ( ! $standard ? sanitize_title( array_shift( $all_rates_class ) ) : 'standard-rate' );

		return array(
			'taxes'     => $taxes,
			'tax_apply' => $tax_apply,
		);
	}

	public static function get_info_taxes_include_in_cart( $apply_currency, $shipping_total ) {
		$cart_contents = WC()->cart->get_cart_contents();
		$taxes_info    = self::get_taxes_in_cart( $apply_currency, $cart_contents );
		if ( isset( $taxes_info['taxes'] ) ) {
			$taxes                = $taxes_info['taxes'];
			$total_tax            = 0;
			$total_coupon_applies = 0;
			$total_tax_fees       = self::get_total_fees( $apply_currency );
			foreach ( $taxes as $tax_key => $tax_value ) {
				$total_coupon_applies = WC()->cart->applied_coupons ? self::calculate_coupon_by_product_in_cart( $apply_currency, $tax_value ) : 0;
				$tax_subtotal         = $tax_value['subtotal'] - $total_coupon_applies + $total_tax_fees;
				$tax_percent          = $tax_value['tax_info']['tax_value'] / 100;
				if ( $taxes_info['tax_apply'] === $tax_key ) {
					$tax_rate_shipping = isset( $tax_value['tax_info']['tax_rate']['tax_rate_shipping'] ) ? (int) $tax_value['tax_info']['tax_rate']['tax_rate_shipping'] : false;
					if ( $tax_value['subtotal'] < $total_coupon_applies ) {
						$total_by_tax = $tax_rate_shipping ? $shipping_total * $tax_percent : 0;
					} else {
						$total_by_tax = $tax_rate_shipping ? ( $tax_subtotal + $shipping_total ) * $tax_percent : $tax_subtotal * $tax_percent;
					}
				} else {
					$total_by_tax = $tax_subtotal * $tax_percent;
				}
				$total_tax                     = $total_tax + $total_by_tax;
				$taxes[ $tax_key ]['subtotal'] = $total_by_tax;
			}

			return array(
				'taxes'     => $taxes,
				'tax_apply' => $taxes_info['tax_apply'],
				'total_tax' => $total_tax,
			);
		}

		return false;

	}

	public static function get_cart_subtotal_by_fixed_include_tax( $apply_currency ) {
		$shipping_total = self::get_shipping_total_selected( $apply_currency );
		$taxes_in_cart  = self::get_info_taxes_include_in_cart( $apply_currency, $shipping_total );
		$subtotal       = $taxes_in_cart ? $taxes_in_cart['total_tax'] : 0;
		return $subtotal;
	}

	public static function get_shipping_flat_rate_fee_total_selected( $apply_currency, $caculate_total ) {
		$shipping = WC()->session->get( 'shipping_for_package_0' );
		if ( ! $shipping || ! isset( $shipping['rates'] ) ) {
			return false;
		}
		$flag = false;
		foreach ( $shipping['rates'] as $method_id => $rate ) {
			if ( WC()->session->get( 'chosen_shipping_methods' )[0] == $method_id ) {

				if ( 'local_pickup' == $rate->method_id ) {
					$shipping = new \WC_Shipping_Local_Pickup( $rate->instance_id );
					if ( ! $caculate_total && 'taxable' != $shipping->tax_status ) {
						$flag = -1;
						break;
					}
				}

				if ( 'flat_rate' == $rate->method_id ) {
					$shipping = new \WC_Shipping_Flat_Rate( $rate->instance_id );
					if ( ! $caculate_total && 'taxable' != $shipping->tax_status ) {
						$flag = -1;
						break;
					}
					$yay_shipping_flat_rate = new ShippingFlatRate();
					$cost                   = $shipping->get_option( 'cost' );
					if ( ! empty( $cost ) && ! is_numeric( $cost ) && FixedPriceHelper::check_product_in_cart_fixed_price( $apply_currency ) ) {
						$args                = array(
							'qty'  => FixedPriceHelper::get_package_item_qty(),
							'cost' => apply_filters( 'yay_currency_checkout_get_cart_subtotal_fixed', 0, $apply_currency ),
						);
						$cost_caculate_again = $yay_shipping_flat_rate->evaluate_cost( $cost, $args, true );
						$flag                = YayCurrencyHelper::calculate_price_by_currency( $cost_caculate_again, false, $apply_currency );

						break;
					}
				}
			}
		}
		return $flag;
	}


	public static function get_total_fees( $apply_currency, $caculate_cart_total = false ) {
		$total_fees = 0;
		foreach ( WC()->cart->get_fees() as $fee ) {

			if ( $fee->taxable || $caculate_cart_total ) {
				$total_fees += YayCurrencyHelper::calculate_price_by_currency( $fee->amount, false, $apply_currency );
			}
		}
		return $total_fees;
	}

	public static function get_shipping_total_selected( $apply_currency, $caculate_total = false ) {

		$shipping_total          = YayCurrencyHelper::calculate_price_by_currency( WC()->cart->shipping_total, false, $apply_currency );
		$shipping_flat_fee_total = self::get_shipping_flat_rate_fee_total_selected( $apply_currency, $caculate_total );

		if ( $shipping_flat_fee_total ) {
			$shipping_total = -1 == $shipping_flat_fee_total ? 0 : $shipping_flat_fee_total;
		}

		return $shipping_total;

	}

	public static function get_price_by_tax_class( $apply_currency, $tax_rate_id ) {
		$tax_rate       = \WC_Tax::_get_tax_rate( $tax_rate_id );
		$tax_class      = isset( $tax_rate['tax_rate_class'] ) && ! empty( $tax_rate['tax_rate_class'] ) ? $tax_rate['tax_rate_class'] : 'standard-rate';
		$shipping_total = self::get_shipping_total_selected( $apply_currency );
		$taxes_in_cart  = self::get_info_taxes_include_in_cart( $apply_currency, $shipping_total );
		$price          = $taxes_in_cart ? ( isset( $taxes_in_cart['taxes'][ $tax_class ] ) ? $taxes_in_cart['taxes'][ $tax_class ]['subtotal'] : 0 ) : 0;
		return $price;
	}

	public static function get_total_coupon_applies( $apply_currency, $product_subtotal, $quantity_product, $tax_products = array() ) {
		$total_coupon_applies = 0;
		if ( WC()->cart->applied_coupons ) {
			$applied_coupons = WC()->cart->applied_coupons;
			foreach ( $applied_coupons  as $coupon_code ) {
				$coupon          = new \WC_Coupon( $coupon_code );
				$discount_type   = $coupon->get_discount_type();
				$discount_amount = (float) $coupon->get_data()['amount'];
				$coupon_data     = $coupon->get_data();
				$products_ids    = isset( $coupon_data['product_ids'] ) && $coupon_data['product_ids'] ? $coupon_data['product_ids'] : false;
				if ( SupportHelper::woo_discount_rules_active() && $products_ids ) {
					if ( 'percent' === $discount_type ) {
						$cart_subtotal_apply_discount     = YayCurrencyHelper::calculate_price_by_currency( $discount_amount, false, $apply_currency );
						$cart_subtotal_apply_rule_as_cart = apply_filters( 'yay_currency_discount_rules_get_cart_subtotal_apply_coupon_as_cart_rule', $cart_subtotal_apply_discount, $discount_amount, $products_ids, $apply_currency );
						$total_coupon_applies            += $cart_subtotal_apply_rule_as_cart;
					}
				} else {
					if ( 'percent' !== $discount_type ) {
						if ( 'fixed_product' === $discount_type ) {
							$discount_amount *= $quantity_product;
						}
						$cart_subtotal_apply_discount = YayCurrencyHelper::calculate_price_by_currency( $discount_amount, false, $apply_currency );

						if ( self::check_product_fixed_price_in_tax_class( $apply_currency, $tax_products ) ) {
							$cart_subtotal_fixed_apply_discount = self::get_tax_product_subtotal_fixed_apply_discount( $apply_currency, $tax_products, (float) $discount_amount );
							$cart_subtotal_fixed_apply_discount = $cart_subtotal_fixed_apply_discount > $cart_subtotal_apply_discount ? $cart_subtotal_apply_discount : $cart_subtotal_fixed_apply_discount;
							$total_coupon_applies              += $cart_subtotal_fixed_apply_discount;
						} else {
							$total_coupon_applies += $cart_subtotal_apply_discount;
						}
					} else {
						$total_coupon_applies += ( $product_subtotal * $discount_amount ) / 100;
					}
				}
			}
		}
		return $total_coupon_applies;
	}

	public static function calculate_coupon_by_product_in_cart( $apply_currency, $tax_details ) {
		$total_coupon_applies = self::get_total_coupon_applies( $apply_currency, $tax_details['subtotal'], $tax_details['quantity_product'], $tax_details['products'] );
		return $total_coupon_applies;
	}

	public static function check_product_fixed_price_in_tax_class( $apply_currency, $tax_products ) {
		if ( ! $tax_products ) {
			return FixedPriceHelper::check_product_in_cart_fixed_price( $apply_currency );
		}
		$flag = false;
		foreach ( $tax_products  as $key => $product ) {
			$product_id  = $product['product_id'];
			$_product    = wc_get_product( $product_id );
			$fixed_price = FixedPriceHelper::product_is_set_fixed_price_by_currency( $_product, $apply_currency );
			if ( $fixed_price ) {
				$flag = true;
				break;
			}
		}
		return $flag;
	}

	public static function get_tax_product_subtotal_fixed_apply_discount( $apply_currency, $tax_products, $coupon_amount ) {
		$cart_subtotal_fixed = 0;
		if ( $tax_products ) {
			foreach ( $tax_products  as $key => $product ) {
				$product_id          = $product['product_id'];
				$_product            = wc_get_product( $product_id );
				$fixed_product_price = FixedPriceHelper::product_is_set_fixed_price_by_currency( $_product, $apply_currency );
				if ( $fixed_product_price ) {
					$cart_subtotal_fixed = $cart_subtotal_fixed + ( $fixed_product_price * $product['quantity'] );
				} else {
					$convert_coupon_amount = YayCurrencyHelper::calculate_price_by_currency( $coupon_amount, false, $apply_currency );
					$cart_subtotal_fixed   = $cart_subtotal_fixed + $convert_coupon_amount * $product['quantity'];
				}
			}
		} else {
			$cart_subtotal_fixed = FixedPriceHelper::get_cart_subtotal_fixed_apply_discount( $apply_currency, (float) $coupon_amount );
		}
		return $cart_subtotal_fixed;
	}
}
